#pragma once
#include "ProcessNotification.h"


/****************************************************************************
*                             class ChildProcess
****************************************************************************/

class ChildProcess {
    public:
       ChildProcess() { Clear(); }
       BOOL setup;
       HWND hWnd;
    protected:
       void Clear() { setup = FALSE; hWnd = NULL; }
};

/****************************************************************************
*                             class ChildProcesses
****************************************************************************/

class ChildProcesses {
    public:

       /****************************************************************************
       *                             ChildProcesses::Add
       * Inputs:
       *       ChildProcess proc: A child process object
       * Result: INT_PTR
       *       The index of where it was added
       * Effect: 
       *       Adds the ChildProcess object to the list of child processes
       ****************************************************************************/

       INT_PTR Add(ChildProcess proc) { return children.Add(proc); }


       /****************************************************************************
       *                          ChildProcesses::RemoveAt
       * Inputs:
       *       INT_PTR i: index of the object to be removed
       * Result: void
       *       
       * Effect: 
       *       Removes the object at the specified index
       ****************************************************************************/
       
       void RemoveAt(INT_PTR i)  { children.RemoveAt(i); }


       /****************************************************************************
       *                           ChildProcesses::Remove
       * Inputs:
       *       HWND child: Child window ID of a child process
       * Result: void
       *       
       * Effect: 
       *       Deletes the matching child process
       * Notes:
       *       There is not check to see if the window actually exists in the
       *       table
       ****************************************************************************/
       
       void Remove(HWND child) { for(INT_PTR i = 1; i < children.GetSize(); i++)
                                   if(children[i].hWnd == child)
                                      { /* delete it */
                                       children.RemoveAt(i);
                                       break;
                                      } /* delete it */
                               }

       /****************************************************************************
       *                          ChildProcesses::ResetAll
       * Result: void
       *       
       * Effect: 
       *       Marks all child processes as being in the not-setup state
       ****************************************************************************/

       void ResetAll() { for(INT_PTR i = 0; i < children.GetSize(); i++) children[i].setup = FALSE; }

       /****************************************************************************
       *                         ChildProcesses::SendMessage
       *                         ChildProcesses::PostMessage
       * Inputs:
       *       UINT msg: Message to send or post
       *       WPARAM wParam = 0: Desired WPARAM
       *       LPARAM lParam = 0: Desired LPARAM
       * Result: void
       *       
       * Effect: 
       *       Sends or posts a message to all child windows
       * Notes:
       *       The value of SendMessage is ignored.
       ****************************************************************************/
       
       void SendMessage(UINT msg, WPARAM wParam = 0, LPARAM lParam = 0) {
             for(INT_PTR i = 0; i < children.GetSize(); i++)
                ::SendMessage(children[i].hWnd, msg, wParam, lParam);
             }

       void PostMessage(UINT msg, WPARAM wParam = 0, LPARAM lParam = 0) {
             for(INT_PTR i = 0; i < children.GetSize(); i++)
                ::PostMessage(children[i].hWnd, msg, wParam, lParam);
            }

       /****************************************************************************
       *                           ChildProcesses::GetSize
       * Result: INT_PTR
       *       The number of child processes in the table
       ****************************************************************************/

       INT_PTR GetSize() { return children.GetSize(); }

       /****************************************************************************
       *                          ChildProcesses::AllSetUp
       * Result: BOOL
       *       TRUE if all child processes have achieved the "setup" state
       *        FALSE if any child process has not achieved the "setup" state
       ****************************************************************************/

       BOOL AllSetUp() {
          BOOL All = TRUE;
          for(INT_PTR i = 0; i < children.GetSize(); i++)
             All &= children[i].setup;
          return All;
         }

       /****************************************************************************
       *                         ChildProcesses::MarkAsSetup
       * Inputs:
       *       HWND child: the child HWND to check
       * Result: void
       *       
       * Effect: 
       *       Marks the child associated with 'child' as having achieved the
       *       setup state
       ****************************************************************************/

       void MarkAsSetup(HWND child) {
          for(INT_PTR i = 0; i < children.GetSize(); i++)
             if(children[i].hWnd == child)
                { /* got it */
                 children[i].setup = TRUE;
                 break;
                } /* got it */
           }

       /****************************************************************************
       *                          ChildProcesses::Validate
       * Inputs:
       *       CWnd * parent: Window to notify
       * Result: void
       *       
       * Effect: 
       *       Makes sure each child is still valid; if the window has closed,
       *       sends a notification to the parent
       ****************************************************************************/

       void Validate(CWnd * parent) {
          for(INT_PTR i = 0; i < children.GetSize(); i++)
             { /* check each */
              if(!::IsWindow(children[i].hWnd))
                 { /* delete it */
                  parent->PostMessage(UWM_PROCESS_NOTIFICATION, FALSE, (LPARAM)children[i].hWnd);
                 } /* delete it */
             } /* check each */
       }
       //***************************************************************************
    protected:
       CArray<ChildProcess, ChildProcess&> children;
};
